@@ -2,7 +2,7 @@
  * mono.c : stereo2mono downmixsimple channel mixer plug-in
  *****************************************************************************
  * Copyright (C) 2006 M2X
- * $Id: 0bb71ccd33ff7881cdb70dc638588c54adea0fd6 $
+ * $Id: 45b64b724ae6c27971dd699a80f821c6b8d9c283 $
  *
  * Authors: Jean-Paul Saman <jpsaman at m2x dot nl>
  *
@@ -31,6 +31,7 @@
 #include <math.h>                                        /* sqrt */
 #include <stdint.h>                                         /* int16_t .. */
 
+#define VLC_MODULE_LICENSE VLC_LICENSE_GPL_2_PLUS
 #include <vlc_common.h>
 #include <vlc_plugin.h>
 #include <vlc_block.h>
@@ -347,9 +348,6 @@ static int OpenFilter( vlc_object_t *p_this )
         return VLC_EGENERIC;
     }
 
-    p_filter->fmt_in.audio.i_format = VLC_CODEC_S16N;
-    p_filter->fmt_out.audio.i_format = VLC_CODEC_S16N;
-
     /* Allocate the memory needed to store the module's structure */
     p_sys = p_filter->p_sys = malloc( sizeof(filter_sys_t) );
     if( p_sys == NULL )
@@ -398,6 +396,11 @@ static int OpenFilter( vlc_object_t *p_this )
              p_filter->fmt_in.audio.i_bitspersample,
              p_filter->fmt_out.audio.i_bitspersample );
 
+    p_filter->fmt_in.audio.i_format = VLC_CODEC_S16N;
+    aout_FormatPrepare(&p_filter->fmt_in.audio);
+    p_filter->fmt_out.audio.i_format = VLC_CODEC_S16N;
+    aout_FormatPrepare(&p_filter->fmt_out.audio);
+
     return VLC_SUCCESS;
 }
 
