@@ -18,9 +18,9 @@
  * Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
-#if !defined(_WIN32_WINNT) || _WIN32_WINNT < 0x600
+#if !defined(_WIN32_WINNT) || _WIN32_WINNT < _WIN32_WINNT_VISTA
 # undef _WIN32_WINNT
-# define _WIN32_WINNT 0x600
+# define _WIN32_WINNT _WIN32_WINNT_VISTA
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -30,18 +30,48 @@
 #define INITGUID
 #define COBJMACROS
 #define CONST_VTABLE
+#define NONEWWAVE
 
 #include <stdlib.h>
 #include <assert.h>
-#include <audioclient.h>
 
 #include <vlc_common.h>
+#include <vlc_codecs.h>
 #include <vlc_aout.h>
 #include <vlc_plugin.h>
+
+#include <audioclient.h>
 #include "audio_output/mmdevice.h"
 
-DEFINE_GUID(_KSDATAFORMAT_SUBTYPE_IEC61937_DOLBY_DIGITAL, WAVE_FORMAT_DOLBY_AC3_SPDIF, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71);
-static const GUID __KSDATAFORMAT_SUBTYPE_IEC61937_DOLBY_DIGITAL = {WAVE_FORMAT_DOLBY_AC3_SPDIF, 0x0000, 0x0010, {0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71}};
+/* 00000092-0000-0010-8000-00aa00389b71 */
+DEFINE_GUID(_KSDATAFORMAT_SUBTYPE_IEC61937_DOLBY_DIGITAL,
+            WAVE_FORMAT_DOLBY_AC3_SPDIF, 0x0000, 0x0010, 0x80, 0x00,
+            0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71);
+
+/* 00000001-0000-0010-8000-00aa00389b71 */
+DEFINE_GUID(_KSDATAFORMAT_SUBTYPE_WAVEFORMATEX,
+            WAVE_FORMAT_PCM, 0x0000, 0x0010, 0x80, 0x00,
+            0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71);
+
+/* 00000008-0000-0010-8000-00aa00389b71 */
+DEFINE_GUID(_KSDATAFORMAT_SUBTYPE_IEC61937_DTS,
+            WAVE_FORMAT_DTS_MS, 0x0000, 0x0010, 0x80, 0x00,
+            0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71);
+
+/* 0000000b-0cea-0010-8000-00aa00389b71 */
+DEFINE_GUID(_KSDATAFORMAT_SUBTYPE_IEC61937_DTS_HD,
+            0x000b, 0x0cea, 0x0010, 0x80, 0x00,
+            0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71);
+
+/* 0000000a-0cea-0010-8000-00aa00389b71 */
+DEFINE_GUID(_KSDATAFORMAT_SUBTYPE_IEC61937_DOLBY_DIGITAL_PLUS,
+            0x000a, 0x0cea, 0x0010, 0x80, 0x00,
+            0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71);
+
+/* 0000000c-0cea-0010-8000-00aa00389b71 */
+DEFINE_GUID(_KSDATAFORMAT_SUBTYPE_IEC61937_DOLBY_MLP,
+            0x000c, 0x0cea, 0x0010, 0x80, 0x00,
+            0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71);
 
 static BOOL CALLBACK InitFreq(INIT_ONCE *once, void *param, void **context)
 {
@@ -71,8 +101,7 @@ typedef struct aout_stream_sys
 
     vlc_fourcc_t format; /**< Sample format */
     unsigned rate; /**< Sample rate */
-    unsigned bytes_per_frame;
-    unsigned frame_length;
+    unsigned block_align;
     UINT64 written; /**< Frames written to the buffer */
     UINT32 frames; /**< Total buffer size (frames) */
 } aout_stream_sys_t;
@@ -159,7 +188,7 @@ static HRESULT Play(aout_stream_t *s, block_t *block)
             break;
         }
 
-        const size_t copy = frames * sys->bytes_per_frame / sys->frame_length;
+        const size_t copy = frames * sys->block_align;
 
         memcpy(dst, block->p_buffer, copy);
         hr = IAudioRenderClient_ReleaseBuffer(render, frames, 0);
@@ -235,15 +264,74 @@ static const uint32_t chans_in[] = {
     SPEAKER_FRONT_CENTER, SPEAKER_LOW_FREQUENCY, 0
 };
 
+static void vlc_HdmiToWave(WAVEFORMATEXTENSIBLE_IEC61937 *restrict wf_iec61937,
+                           audio_sample_format_t *restrict audio)
+{
+    WAVEFORMATEXTENSIBLE *wf = &wf_iec61937->FormatExt;
+
+    switch (audio->i_format)
+    {
+    case VLC_CODEC_DTS:
+        wf->SubFormat = _KSDATAFORMAT_SUBTYPE_IEC61937_DTS_HD;
+        wf->Format.nChannels = 8;
+        wf->dwChannelMask = KSAUDIO_SPEAKER_7POINT1;
+        audio->i_rate = 768000;
+        break;
+    case VLC_CODEC_EAC3:
+        wf->SubFormat = _KSDATAFORMAT_SUBTYPE_IEC61937_DOLBY_DIGITAL_PLUS;
+        wf->Format.nChannels = 2;
+        wf->dwChannelMask = KSAUDIO_SPEAKER_5POINT1;
+        break;
+    case VLC_CODEC_TRUEHD:
+    case VLC_CODEC_MLP:
+        wf->SubFormat = _KSDATAFORMAT_SUBTYPE_IEC61937_DOLBY_MLP;
+        wf->Format.nChannels = 8;
+        wf->dwChannelMask = KSAUDIO_SPEAKER_7POINT1;
+        audio->i_rate = 768000;
+        break;
+    default:
+        vlc_assert_unreachable();
+    }
+    wf->Format.wFormatTag = WAVE_FORMAT_EXTENSIBLE;
+    wf->Format.nSamplesPerSec = 192000;
+    wf->Format.wBitsPerSample = 16;
+    wf->Format.nBlockAlign = wf->Format.wBitsPerSample / 8 * wf->Format.nChannels;
+    wf->Format.nAvgBytesPerSec = wf->Format.nSamplesPerSec * wf->Format.nBlockAlign;
+    wf->Format.cbSize = sizeof (*wf_iec61937) - sizeof (wf->Format);
+
+    wf->Samples.wValidBitsPerSample = wf->Format.wBitsPerSample;
+
+    wf_iec61937->dwEncodedSamplesPerSec = audio->i_rate;
+    wf_iec61937->dwEncodedChannelCount = audio->i_channels;
+    wf_iec61937->dwAverageBytesPerSec = 0;
+
+    audio->i_format = VLC_CODEC_SPDIFL;
+    audio->i_bytes_per_frame = wf->Format.nBlockAlign;
+    audio->i_frame_length = 1;
+}
+
 static void vlc_SpdifToWave(WAVEFORMATEXTENSIBLE *restrict wf,
                             audio_sample_format_t *restrict audio)
 {
-    audio->i_format = VLC_CODEC_SPDIFL;
-    aout_FormatPrepare (audio);
-    audio->i_bytes_per_frame = AOUT_SPDIF_SIZE;
-    audio->i_frame_length = A52_FRAME_NB;
-
-    wf->SubFormat = _KSDATAFORMAT_SUBTYPE_IEC61937_DOLBY_DIGITAL;
+    switch (audio->i_format)
+    {
+    case VLC_CODEC_DTS:
+        if (audio->i_rate < 48000)
+        {
+            /* Wasapi doesn't accept DTS @ 44.1kHz but accept IEC 60958 PCM */
+            wf->SubFormat = _KSDATAFORMAT_SUBTYPE_WAVEFORMATEX;
+        }
+        else
+            wf->SubFormat = _KSDATAFORMAT_SUBTYPE_IEC61937_DTS;
+        break;
+    case VLC_CODEC_SPDIFL:
+    case VLC_CODEC_SPDIFB:
+    case VLC_CODEC_A52:
+        wf->SubFormat = _KSDATAFORMAT_SUBTYPE_IEC61937_DOLBY_DIGITAL;
+        break;
+    default:
+        vlc_assert_unreachable();
+    }
 
     wf->Format.wFormatTag = WAVE_FORMAT_EXTENSIBLE;
     wf->Format.nChannels = 2; /* To prevent channel re-ordering */
@@ -256,6 +344,10 @@ static void vlc_SpdifToWave(WAVEFORMATEXTENSIBLE *restrict wf,
     wf->Samples.wValidBitsPerSample = wf->Format.wBitsPerSample;
 
     wf->dwChannelMask = SPEAKER_FRONT_LEFT | SPEAKER_FRONT_RIGHT;
+
+    audio->i_format = VLC_CODEC_SPDIFL;
+    audio->i_bytes_per_frame = wf->Format.nBlockAlign;
+    audio->i_frame_length = 1;
 }
 
 static void vlc_ToWave(WAVEFORMATEXTENSIBLE *restrict wf,
@@ -347,7 +439,6 @@ static int vlc_FromWave(const WAVEFORMATEX *restrict wf,
     else
         return -1;
 
-    audio->i_original_channels = audio->i_physical_channels;
     aout_FormatPrepare (audio);
 
     if (wf->nChannels != audio->i_channels)
@@ -369,7 +460,8 @@ static unsigned vlc_CheckWaveOrder (const WAVEFORMATEX *restrict wf,
     return aout_CheckChannelReorder(chans_in, chans_out, mask, table);
 }
 
-static HRESULT Start(aout_stream_t *s, audio_sample_format_t *restrict fmt,
+
+static HRESULT Start(aout_stream_t *s, audio_sample_format_t *restrict pfmt,
                      const GUID *sid)
 {
     static INIT_ONCE freq_once = INIT_ONCE_STATIC_INIT;
@@ -382,6 +474,27 @@ static HRESULT Start(aout_stream_t *s, audio_sample_format_t *restrict fmt,
         return E_OUTOFMEMORY;
     sys->client = NULL;
 
+    /* Configure audio stream */
+    WAVEFORMATEXTENSIBLE_IEC61937 wf_iec61937;
+    WAVEFORMATEXTENSIBLE *pwfe = &wf_iec61937.FormatExt;
+    WAVEFORMATEX *pwf = &pwfe->Format, *pwf_closest, *pwf_mix = NULL;
+    AUDCLNT_SHAREMODE shared_mode;
+    REFERENCE_TIME buffer_duration;
+    audio_sample_format_t fmt = *pfmt;
+    bool b_spdif = AOUT_FMT_SPDIF(&fmt);
+    bool b_hdmi = AOUT_FMT_HDMI(&fmt);
+    bool b_dtshd = false;
+
+    if (fmt.i_format == VLC_CODEC_DTS)
+    {
+        b_dtshd = var_GetBool(s->obj.parent, "dtshd");
+        if (b_dtshd)
+        {
+            b_hdmi = true;
+            b_spdif = false;
+        }
+    }
+
     void *pv;
     HRESULT hr = aout_stream_Activate(s, &IID_IAudioClient, NULL, &pv);
     if (FAILED(hr))
@@ -391,62 +504,98 @@ static HRESULT Start(aout_stream_t *s, audio_sample_format_t *restrict fmt,
     }
     sys->client = pv;
 
-    /* Configure audio stream */
-    WAVEFORMATEXTENSIBLE wf;
-    WAVEFORMATEX *pwf;
-    AUDCLNT_SHAREMODE shared_mode;
-
-    if (AOUT_FMT_SPDIF(fmt) && var_InheritBool(s, "spdif"))
+    if (b_spdif)
     {
-        vlc_SpdifToWave(&wf, fmt);
+        vlc_SpdifToWave(pwfe, &fmt);
         shared_mode = AUDCLNT_SHAREMODE_EXCLUSIVE;
+        /* The max buffer duration in exclusive mode is 2 seconds */
+        buffer_duration = AOUT_MAX_PREPARE_TIME;
+    }
+    else if (b_hdmi)
+    {
+        vlc_HdmiToWave(&wf_iec61937, &fmt);
+        shared_mode = AUDCLNT_SHAREMODE_EXCLUSIVE;
+        /* The max buffer duration in exclusive mode is 2 seconds */
+        buffer_duration = AOUT_MAX_PREPARE_TIME;
+    }
+    else if (AOUT_FMT_LINEAR(&fmt))
+    {
+        shared_mode = AUDCLNT_SHAREMODE_SHARED;
+
+        if (fmt.channel_type == AUDIO_CHANNEL_TYPE_AMBISONICS)
+        {
+            fmt.channel_type = AUDIO_CHANNEL_TYPE_BITMAP;
+
+            /* Render Ambisonics on the native mix format */
+            hr = IAudioClient_GetMixFormat(sys->client, &pwf_mix);
+            if (FAILED(hr) || vlc_FromWave(pwf_mix, &fmt))
+                vlc_ToWave(pwfe, &fmt); /* failed, fallback to default */
+            else
+                pwf = pwf_mix;
+
+            /* Setup low latency in order to quickly react to ambisonics filters
+             * viewpoint changes. */
+            buffer_duration = AOUT_MIN_PREPARE_TIME;
+        }
+        else
+        {
+            vlc_ToWave(pwfe, &fmt);
+            buffer_duration = AOUT_MAX_PREPARE_TIME * 10;
+        }
     }
     else
     {
-        vlc_ToWave(&wf, fmt);
-        shared_mode = AUDCLNT_SHAREMODE_SHARED;
+        hr = E_FAIL;
+        goto error;
     }
 
     hr = IAudioClient_IsFormatSupported(sys->client, shared_mode,
-                                        &wf.Format, &pwf);
-    if (FAILED(hr) && AOUT_FMT_SPDIF(fmt))
-    {
-        /* Device may not support SPDIF: try again with FL32 */
-        vlc_ToWave(&wf, fmt);
-        shared_mode = AUDCLNT_SHAREMODE_SHARED;
-        hr = IAudioClient_IsFormatSupported(sys->client, shared_mode,
-                                            &wf.Format, &pwf);
-    }
+                                        pwf, &pwf_closest);
+
     if (FAILED(hr))
     {
-        msg_Err(s, "cannot negotiate audio format (error 0x%lx)", hr);
+        if (pfmt->i_format == VLC_CODEC_DTS && b_hdmi)
+        {
+            msg_Warn(s, "cannot negotiate DTS at 768khz IEC958 rate (HDMI), "
+                     "fallback to 48kHz (S/PDIF) (error 0x%lx)", hr);
+            IAudioClient_Release(sys->client);
+            free(sys);
+            var_SetBool(s->obj.parent, "dtshd", false);
+            return Start(s, pfmt, sid);
+        }
+        msg_Err(s, "cannot negotiate audio format (error 0x%lx)%s", hr,
+                hr == AUDCLNT_E_UNSUPPORTED_FORMAT
+                && fmt.i_format == VLC_CODEC_SPDIFL ?
+                ": digital pass-through not supported" : "");
         goto error;
     }
 
     if (hr == S_FALSE)
     {
-        assert(pwf != NULL);
-        if (vlc_FromWave(pwf, fmt))
+        assert(pwf_closest != NULL);
+        if (vlc_FromWave(pwf_closest, &fmt))
         {
-            CoTaskMemFree(pwf);
+            CoTaskMemFree(pwf_closest);
             msg_Err(s, "unsupported audio format");
             hr = E_INVALIDARG;
             goto error;
         }
         shared_mode = AUDCLNT_SHAREMODE_SHARED;
         msg_Dbg(s, "modified format");
+        pwf = pwf_closest;
     }
     else
-        assert(pwf == NULL);
+        assert(pwf_closest == NULL);
 
-    sys->chans_to_reorder = vlc_CheckWaveOrder((hr == S_OK) ? &wf.Format : pwf,
-                                               sys->chans_table);
-    sys->format = fmt->i_format;
+    sys->chans_to_reorder = fmt.i_format != VLC_CODEC_SPDIFL ?
+                            vlc_CheckWaveOrder(pwf, sys->chans_table) : 0;
+    sys->format = fmt.i_format;
+    sys->block_align = pwf->nBlockAlign;
+    sys->rate = pwf->nSamplesPerSec;
 
-    hr = IAudioClient_Initialize(sys->client, shared_mode, 0,
-                                 AOUT_MAX_PREPARE_TIME * 10, 0,
-                                 (hr == S_OK) ? &wf.Format : pwf, sid);
-    CoTaskMemFree(pwf);
+    hr = IAudioClient_Initialize(sys->client, shared_mode, 0, buffer_duration,
+                                 0, pwf, sid);
+    CoTaskMemFree(pwf_closest);
     if (FAILED(hr))
     {
         msg_Err(s, "cannot initialize audio client (error 0x%lx)", hr);
@@ -470,9 +619,8 @@ static HRESULT Start(aout_stream_t *s, audio_sample_format_t *restrict fmt,
         msg_Dbg(s, "minimum period : %"PRIu64"00 ns", minT);
     }
 
-    sys->rate = fmt->i_rate;
-    sys->bytes_per_frame = fmt->i_bytes_per_frame;
-    sys->frame_length = fmt->i_frame_length > 0 ? fmt->i_frame_length : 1;
+    CoTaskMemFree(pwf_mix);
+    *pfmt = fmt;
     sys->written = 0;
     s->sys = sys;
     s->time_get = TimeGet;
@@ -481,6 +629,7 @@ static HRESULT Start(aout_stream_t *s, audio_sample_format_t *restrict fmt,
     s->flush = Flush;
     return S_OK;
 error:
+    CoTaskMemFree(pwf_mix);
     if (sys->client != NULL)
         IAudioClient_Release(sys->client);
     free(sys);
