@@ -220,7 +220,7 @@ static void CSDFree(decoder_t *p_dec)
 static void CSDInit(decoder_t *p_dec, block_t *p_blocks, size_t i_count)
 {
     decoder_sys_t *p_sys = p_dec->p_sys;
-    assert(i_count >= 0 && i_count <= 3);
+    assert(i_count <= MAX_CSD_COUNT);
 
     CSDFree(p_dec);
 
@@ -692,7 +692,7 @@ static int OpenDecoder(vlc_object_t *p_this, pf_MediaCodecApi_init pf_init)
 
             if (p_sys->api.b_support_rotation)
             {
-                switch (p_dec->fmt_out.video.orientation)
+                switch (p_dec->fmt_in.video.orientation)
                 {
                     case ORIENT_ROTATED_90:
                         p_sys->video.i_angle = 90;
