@@ -27,6 +27,7 @@
 #include <vlc_stream.h>
 #include <vlc_demux.h>
 #include "SourceStream.hpp"
+#include "../StreamFormat.hpp"
 #include "CommandsQueue.hpp"
 #include "../ChunksSource.hpp"
 
@@ -35,7 +36,9 @@ using namespace adaptive;
 AbstractDemuxer::AbstractDemuxer()
 {
     b_startsfromzero = false;
-    b_reinitsonseek =true;
+    b_reinitsonseek = true;
+    b_candetectswitches = true;
+    b_alwaysrestarts = false;
 }
 
 AbstractDemuxer::~AbstractDemuxer()
@@ -48,11 +51,93 @@ bool AbstractDemuxer::alwaysStartsFromZero() const
     return b_startsfromzero;
 }
 
-bool AbstractDemuxer::reinitsOnSeek() const
+bool AbstractDemuxer::needsRestartOnSwitch() const
+{
+    return !b_candetectswitches;
+}
+
+bool AbstractDemuxer::needsRestartOnEachSegment() const
+{
+    return b_alwaysrestarts;
+}
+
+void AbstractDemuxer::setCanDetectSwitches( bool b )
+{
+    b_candetectswitches = b;
+}
+
+void AbstractDemuxer::setRestartsOnEachSegment( bool b )
+{
+    b_alwaysrestarts = b;
+}
+
+bool AbstractDemuxer::needsRestartOnSeek() const
 {
     return b_reinitsonseek;
 }
 
+MimeDemuxer::MimeDemuxer(demux_t *p_realdemux_,
+                         const DemuxerFactoryInterface *factory_,
+                         es_out_t *out, AbstractSourceStream *source)
+    : AbstractDemuxer()
+{
+    p_es_out = out;
+    factory = factory_;
+    p_realdemux = p_realdemux_;
+    demuxer = NULL;
+    sourcestream = source;
+}
+
+MimeDemuxer::~MimeDemuxer()
+{
+    if( demuxer )
+        delete demuxer;
+}
+
+bool MimeDemuxer::create()
+{
+    stream_t *p_newstream = sourcestream->makeStream();
+    if(!p_newstream)
+        return false;
+
+    char *type = stream_ContentType(p_newstream);
+    if(type)
+    {
+        demuxer = factory->newDemux( p_realdemux, StreamFormat(std::string(type)),
+                                     p_es_out, sourcestream );
+        free(type);
+    }
+    vlc_stream_Delete(p_newstream);
+
+    if(!demuxer || !demuxer->create())
+        return false;
+
+    return true;
+}
+
+void MimeDemuxer::destroy()
+{
+    if(demuxer)
+    {
+        delete demuxer;
+        demuxer = NULL;
+    }
+    sourcestream->Reset();
+}
+
+void MimeDemuxer::drain()
+{
+    if(demuxer)
+        demuxer->drain();
+}
+
+int MimeDemuxer::demux(mtime_t t)
+{
+    if(!demuxer)
+        return VLC_DEMUXER_EOF;
+    return demuxer->demux(t);
+}
+
 Demuxer::Demuxer(demux_t *p_realdemux_, const std::string &name_, es_out_t *out, AbstractSourceStream *source)
     : AbstractDemuxer()
 {
@@ -65,43 +150,51 @@ Demuxer::Demuxer(demux_t *p_realdemux_, const std::string &name_, es_out_t *out,
 
     if(name == "mp4")
     {
+        b_candetectswitches = false;
         b_startsfromzero = true;
     }
+    else if(name == "aac")
+    {
+        b_candetectswitches = false;
+    }
 }
 
 Demuxer::~Demuxer()
 {
     if(p_demux)
-    {
-        p_demux->s = NULL; // otherwise tries to delete below inner stream
         demux_Delete(p_demux);
-    }
 }
 
 bool Demuxer::create()
 {
+    stream_t *p_newstream = sourcestream->makeStream();
+    if(!p_newstream)
+        return false;
+
     p_demux = demux_New( VLC_OBJECT(p_realdemux), name.c_str(), "",
-                         sourcestream->getStream(), p_es_out );
+                         p_newstream, p_es_out );
     if(!p_demux)
     {
+        vlc_stream_Delete(p_newstream);
         b_eof = true;
         return false;
     }
+    else
+    {
+        b_eof = false;
+    }
+
     return true;
 }
 
-bool Demuxer::restart(CommandsQueue &queue)
+void Demuxer::destroy()
 {
     if(p_demux)
     {
-        queue.setDrop(true);
-        p_demux->s = NULL; // otherwise tries to delete below inner stream
         demux_Delete(p_demux);
         p_demux = NULL;
-        queue.setDrop(false);
     }
     sourcestream->Reset();
-    return create();
 }
 
 void Demuxer::drain()
@@ -146,12 +239,14 @@ bool SlaveDemuxer::create()
 
 int SlaveDemuxer::demux(mtime_t nz_deadline)
 {
-    if( demux_Control(p_demux, DEMUX_SET_NEXT_DEMUX_TIME, VLC_TS_0 + nz_deadline) != VLC_SUCCESS )
+    /* Always call with increment or buffering will get slow stuck */
+    mtime_t i_next_demux_time = VLC_TS_0 + nz_deadline + CLOCK_FREQ / 4;
+    if( demux_Control(p_demux, DEMUX_SET_NEXT_DEMUX_TIME, i_next_demux_time ) != VLC_SUCCESS )
     {
         b_eof = true;
         return VLC_DEMUXER_EOF;
     }
-    int ret = Demuxer::demux(nz_deadline);
-    es_out_Control(p_es_out, ES_OUT_SET_GROUP_PCR, 0, VLC_TS_0 + nz_deadline);
+    int ret = Demuxer::demux(i_next_demux_time);
+    es_out_Control(p_es_out, ES_OUT_SET_GROUP_PCR, 0, i_next_demux_time);
     return ret;
 }
