@@ -25,8 +25,8 @@
 
 using namespace dash;
 
-DASHStream::DASHStream(demux_t *demux, const StreamFormat &format)
-    :AbstractStream(demux, format)
+DASHStream::DASHStream(demux_t *demux)
+    :AbstractStream(demux)
 {
 }
 
@@ -35,25 +35,23 @@ block_t * DASHStream::checkBlock(block_t *p_block, bool)
     return p_block;
 }
 
-AbstractDemuxer * DASHStream::createDemux(const StreamFormat &format)
+AbstractDemuxer *DASHStream::newDemux(demux_t *p_realdemux, const StreamFormat &format,
+                                      es_out_t *out, AbstractSourceStream *source) const
 {
     AbstractDemuxer *ret = NULL;
     switch((unsigned)format)
     {
         case StreamFormat::MP4:
-            ret = new Demuxer(p_realdemux, "mp4", fakeesout->getEsOut(), demuxersource);
-            break;
-
         case StreamFormat::MPEG2TS:
-            ret = new Demuxer(p_realdemux, "ts", fakeesout->getEsOut(), demuxersource);
+            ret = AbstractStream::newDemux(p_realdemux, format, out, source);
             break;
 
         case StreamFormat::WEBVTT:
-            ret = new SlaveDemuxer(p_realdemux, "subtitle", fakeesout->getEsOut(), demuxersource);
+            ret = new SlaveDemuxer(p_realdemux, "webvtt", out, source);
             break;
 
         case StreamFormat::TTML:
-            ret = new SlaveDemuxer(p_realdemux, "ttml", fakeesout->getEsOut(), demuxersource);
+            ret = new SlaveDemuxer(p_realdemux, "ttml", out, source);
             break;
 
         default:
@@ -61,26 +59,17 @@ AbstractDemuxer * DASHStream::createDemux(const StreamFormat &format)
             break;
     }
 
-    if(ret && !ret->create())
-    {
-        delete ret;
-        ret = NULL;
-    }
-    else fakeesout->commandsqueue.Commit();
-
     return ret;
 }
 
 AbstractStream * DASHStreamFactory::create(demux_t *realdemux, const StreamFormat &format,
-                                   SegmentTracker *tracker, HTTPConnectionManager *manager) const
+                                   SegmentTracker *tracker, AbstractConnectionManager *manager) const
 {
-    AbstractStream *stream;
-    try
+    AbstractStream *stream = new (std::nothrow) DASHStream(realdemux);
+    if(stream && !stream->init(format, tracker, manager))
     {
-        stream = new DASHStream(realdemux, format);
-    } catch (int) {
+        delete stream;
         return NULL;
     }
-    stream->bind(tracker, manager);
     return stream;
 }
