@@ -1056,13 +1056,26 @@ int SetupAudioES( demux_t *p_demux, mp4_track_t *p_track, MP4_Box_t *p_sample )
             break;
         }
 
+        case VLC_FOURCC( 't', 'w', 'o', 's' ):
+        case VLC_FOURCC( 's', 'o', 'w', 't' ):
+            p_track->fmt.i_codec = p_sample->i_type;
+            p_track->fmt.audio.i_bitspersample = 16;
+            break;
+
+        case 0x0000000:
         case( VLC_FOURCC( 'r', 'a', 'w', ' ' ) ):
         case( VLC_FOURCC( 'N', 'O', 'N', 'E' ) ):
         {
             if( (p_soun->i_samplesize+7)/8 == 1 )
+            {
                 p_track->fmt.i_codec = VLC_CODEC_U8;
+                p_track->fmt.audio.i_bitspersample = 8;
+            }
             else
+            {
                 p_track->fmt.i_codec = VLC_FOURCC( 't', 'w', 'o', 's' );
+                p_track->fmt.audio.i_bitspersample = 16;
+            }
 
             /* Buggy files workaround */
             if( (p_track->i_timescale != p_soun->i_sampleratehi) )
