@@ -29,9 +29,10 @@
 typedef enum
 {
     TYPE_FREE = 0,
+    TYPE_CAT,
     TYPE_PAT,
     TYPE_PMT,
-    TYPE_PES,
+    TYPE_STREAM,
     TYPE_SI,
     TYPE_PSIP,
 } ts_pid_type_t;
@@ -53,6 +54,7 @@ struct ts_pid_t
 
     uint8_t     i_flags;
     uint8_t     i_cc;   /* countinuity counter */
+    uint8_t     i_dup;  /* duplicate counter */
     uint8_t     type;
 
     uint16_t    i_refcount;
@@ -62,7 +64,7 @@ struct ts_pid_t
     {
         ts_pat_t    *p_pat;
         ts_pmt_t    *p_pmt;
-        ts_pes_t    *p_pes;
+        ts_stream_t    *p_stream;
         ts_si_t     *p_si;
         ts_psip_t   *p_psip;
     } u;
@@ -70,8 +72,10 @@ struct ts_pid_t
     struct
     {
         vlc_fourcc_t i_fourcc;
-        int i_type;
+        vlc_fourcc_t i_original_fourcc;
+        int i_cat;
         int i_pcr_count;
+        uint8_t i_stream_id;
     } probed;
 
 };
