@@ -1,7 +1,7 @@
 /*****************************************************************************
  * AppleRemote.m
  * AppleRemote
- * $Id: 595af50f022f40525bd7866f4d9f0a9a1778e89a $
+ * $Id: 452a1e5d22d75c25827da9273c21e5e3b8a5e995 $
  *
  * Created by Martin Kahr on 11.03.06 under a MIT-style license.
  * Copyright (c) 2006 martinkahr.com. All rights reserved.
@@ -54,7 +54,7 @@
 #import "AppleRemote.h"
 
 /* this was added by the VideoLAN team to ensure Leopard-compatibility and is VLC-only */
-#import "intf.h"
+#import "VLCMain.h"
 #import "CompatibilityFixes.h"
 
 const char* AppleRemoteDeviceName = "AppleIRController";
@@ -89,6 +89,11 @@ const NSTimeInterval HOLD_RECOGNITION_TIME_INTERVAL=0.4;
         [mutableCookieToButtonMapping setObject:[NSNumber numberWithInt:k2009RemoteButtonFullscreen] forKey:@"33_21_20_3_2_33_21_20_3_2_"];
         [mutableCookieToButtonMapping setObject:[NSNumber numberWithInt:kRemoteControl_Switched]     forKey:@"42_33_23_21_20_2_33_23_21_20_2_"];
 
+        if (OSX_HIGH_SIERRA_AND_HIGHER) {
+            [mutableCookieToButtonMapping setObject:[NSNumber numberWithInt:kRemoteButtonVolume_Plus]    forKey:@"33_21_20_15_12_2_"];
+            [mutableCookieToButtonMapping setObject:[NSNumber numberWithInt:kRemoteButtonVolume_Minus]   forKey:@"33_21_20_16_12_2_"];
+        }
+
         _cookieToButtonMapping = [[NSDictionary alloc] initWithDictionary: mutableCookieToButtonMapping];
 
         /* defaults */
