@@ -2,7 +2,7 @@
  * ogg.c: ogg muxer module for vlc
  *****************************************************************************
  * Copyright (C) 2001, 2002, 2006 VLC authors and VideoLAN
- * $Id: 1d57c3f46a47cd5c5b67817c841a0516c77c580c $
+ * $Id: 72d843c7f7fab41b6e31376253296da8a45cb7fa $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Gildas Bazin <gbazin@videolan.org>
@@ -143,8 +143,7 @@ typedef struct
  *****************************************************************************/
 typedef struct
 {
-    enum es_format_category_e i_cat;
-    vlc_fourcc_t i_fourcc;
+    es_format_t fmt;
 
     int b_new;
 
@@ -278,7 +277,6 @@ static void Close( vlc_object_t * p_this )
 {
     sout_mux_t     *p_mux = (sout_mux_t*)p_this;
     sout_mux_sys_t *p_sys = p_mux->p_sys;
-    ogg_stream_t *p_stream;
 
     msg_Info( p_mux, "Close" );
 
@@ -290,10 +288,13 @@ static void Close( vlc_object_t * p_this )
         /* Remove deleted logical streams */
         for(int i = 0; i < p_sys->i_del_streams; i++ )
         {
-            OggCreateStreamFooter( p_mux, p_sys->pp_del_streams[i] );
-            free( p_sys->pp_del_streams[i]->p_oggds_header );
-            free( p_sys->pp_del_streams[i]->skeleton.p_index );
-            free( p_sys->pp_del_streams[i] );
+            ogg_stream_t *p_stream = p_sys->pp_del_streams[i];
+
+            es_format_Clean( &p_stream->fmt );
+            OggCreateStreamFooter( p_mux, p_stream );
+            free( p_stream->p_oggds_header );
+            free( p_stream->skeleton.p_index );
+            free( p_stream );
         }
         free( p_sys->pp_del_streams );
         p_sys->i_streams -= p_sys->i_del_streams;
@@ -353,8 +354,12 @@ static int AddStream( sout_mux_t *p_mux, sout_input_t *p_input )
     if( !p_stream )
         return VLC_ENOMEM;
 
-    p_stream->i_cat       = p_input->p_fmt->i_cat;
-    p_stream->i_fourcc    = p_input->p_fmt->i_codec;
+    if( es_format_Copy( &p_stream->fmt, &p_input->fmt ) != VLC_SUCCESS )
+    {
+        free( p_stream );
+        return VLC_ENOMEM;
+    }
+
     p_stream->i_serial_no = p_sys->i_next_serial_no++;
     p_stream->i_packet_no = 0;
     p_stream->i_last_keyframe = 0;
@@ -371,17 +376,15 @@ static int AddStream( sout_mux_t *p_mux, sout_input_t *p_input )
     {
     case VIDEO_ES:
     {
-        unsigned int i_frame_rate = p_input->p_fmt->video.i_frame_rate;
-        unsigned int i_frame_rate_base = p_input->p_fmt->video.i_frame_rate_base;
-        if( !p_input->p_fmt->video.i_frame_rate ||
-            !p_input->p_fmt->video.i_frame_rate_base )
+        if( !p_stream->fmt.video.i_frame_rate ||
+            !p_stream->fmt.video.i_frame_rate_base )
         {
             msg_Warn( p_mux, "Missing frame rate, assuming 25fps" );
-            i_frame_rate = 25;
-            i_frame_rate_base = 1;
+            p_stream->fmt.video.i_frame_rate = 25;
+            p_stream->fmt.video.i_frame_rate_base = 1;
         }
 
-        switch( p_stream->i_fourcc )
+        switch( p_stream->fmt.i_codec )
         {
         case VLC_CODEC_MP4V:
         case VLC_CODEC_MPGV:
@@ -401,30 +404,30 @@ static int AddStream( sout_mux_t *p_mux, sout_input_t *p_input )
             p_stream->p_oggds_header->i_packet_type = PACKET_TYPE_HEADER;
 
             memcpy( p_stream->p_oggds_header->stream_type, "video", 5 );
-            if( p_stream->i_fourcc == VLC_CODEC_MP4V )
+            if( p_stream->fmt.i_codec == VLC_CODEC_MP4V )
             {
                 memcpy( p_stream->p_oggds_header->sub_type, "XVID", 4 );
             }
-            else if( p_stream->i_fourcc == VLC_CODEC_DIV3 )
+            else if( p_stream->fmt.i_codec == VLC_CODEC_DIV3 )
             {
                 memcpy( p_stream->p_oggds_header->sub_type, "DIV3", 4 );
             }
             else
             {
                 memcpy( p_stream->p_oggds_header->sub_type,
-                        &p_stream->i_fourcc, 4 );
+                        &p_stream->fmt.i_codec, 4 );
             }
             p_stream->p_oggds_header->i_size = 0 ;
             p_stream->p_oggds_header->i_time_unit =
-                     INT64_C(10000000) * i_frame_rate_base /
-                     (int64_t)i_frame_rate;
+                     INT64_C(10000000) * p_stream->fmt.video.i_frame_rate_base /
+                     (int64_t)p_stream->fmt.video.i_frame_rate;
             p_stream->p_oggds_header->i_samples_per_unit = 1;
             p_stream->p_oggds_header->i_default_len = 1 ; /* ??? */
             p_stream->p_oggds_header->i_buffer_size = 1024*1024;
             p_stream->p_oggds_header->i_bits_per_sample = 0;
             p_stream->p_oggds_header->header.video.i_width = p_input->p_fmt->video.i_width;
             p_stream->p_oggds_header->header.video.i_height = p_input->p_fmt->video.i_height;
-            msg_Dbg( p_mux, "%4.4s stream", (char *)&p_stream->i_fourcc );
+            msg_Dbg( p_mux, "%4.4s stream", (char *)&p_stream->fmt.i_codec );
             break;
 
         case VLC_CODEC_DIRAC:
@@ -451,7 +454,7 @@ static int AddStream( sout_mux_t *p_mux, sout_input_t *p_input )
         break;
 
     case AUDIO_ES:
-        switch( p_stream->i_fourcc )
+        switch( p_stream->fmt.i_codec )
         {
         case VLC_CODEC_OPUS:
             msg_Dbg( p_mux, "opus stream" );
@@ -470,7 +473,7 @@ static int AddStream( sout_mux_t *p_mux, sout_input_t *p_input )
             break;
 
         default:
-            fourcc_to_wf_tag( p_stream->i_fourcc, &i_tag );
+            fourcc_to_wf_tag( p_stream->fmt.i_codec, &i_tag );
             if( i_tag == WAVE_FORMAT_UNKNOWN )
             {
                 FREENULL( p_input->p_sys );
@@ -510,13 +513,13 @@ static int AddStream( sout_mux_t *p_mux, sout_input_t *p_input )
             p_stream->p_oggds_header->header.audio.i_channels = p_input->p_fmt->audio.i_channels;
             p_stream->p_oggds_header->header.audio.i_block_align =  p_input->p_fmt->audio.i_blockalign;
             p_stream->p_oggds_header->header.audio.i_avgbytespersec =  p_input->p_fmt->i_bitrate / 8;
-            msg_Dbg( p_mux, "%4.4s stream", (char *)&p_stream->i_fourcc );
+            msg_Dbg( p_mux, "%4.4s stream", (char *)&p_stream->fmt.i_codec );
             break;
         }
         break;
 
     case SPU_ES:
-        switch( p_stream->i_fourcc )
+        switch( p_stream->fmt.i_codec )
         {
         case VLC_CODEC_SUBT:
             p_stream->p_oggds_header = calloc( 1, sizeof(oggds_header_t) );
@@ -737,7 +740,7 @@ static void OggGetSkeletonFisbone( uint8_t **pp_buffer, long *pi_size,
     } headers = { NULL, NULL, 0, 0 };
     *pi_size = 0;
 
-    switch( p_stream->i_fourcc )
+    switch( p_stream->fmt.i_codec )
     {
         case VLC_CODEC_VORBIS:
             psz_value = "audio/vorbis";
@@ -766,7 +769,7 @@ static void OggGetSkeletonFisbone( uint8_t **pp_buffer, long *pi_size,
         default:
             psz_value = "application/octet-stream";
             msg_Warn( p_mux, "Unknown fourcc for stream %s, setting Content-Type to %s",
-                  vlc_fourcc_GetDescription( p_stream->i_cat, p_stream->i_fourcc ),
+                  vlc_fourcc_GetDescription( p_stream->fmt.i_cat, p_stream->fmt.i_codec ),
                   psz_value );
     }
 
@@ -823,8 +826,8 @@ static void OggGetSkeletonFisbone( uint8_t **pp_buffer, long *pi_size,
     switch ( p_input->p_fmt->i_cat )
     {
         case VIDEO_ES:
-            SetQWLE( &(*pp_buffer)[20], p_input->p_fmt->video.i_frame_rate );
-            SetQWLE( &(*pp_buffer)[28], p_input->p_fmt->video.i_frame_rate_base );
+            SetQWLE( &(*pp_buffer)[20], p_stream->fmt.video.i_frame_rate );
+            SetQWLE( &(*pp_buffer)[28], p_stream->fmt.video.i_frame_rate_base );
         break;
         case AUDIO_ES:
             SetQWLE( &(*pp_buffer)[20], p_input->p_fmt->audio.i_rate );
@@ -920,13 +923,15 @@ static int32_t OggFillDsHeader( uint8_t *p_buffer, oggds_header_t *p_oggds_heade
 
 static void OggFillVP8Header( uint8_t *p_buffer, sout_input_t *p_input )
 {
+    ogg_stream_t *p_stream = (ogg_stream_t *) p_input->p_sys;
+
     memcpy( p_buffer, "OVP80\x01\x01\x00", 8 );
     SetWBE( &p_buffer[8], p_input->p_fmt->video.i_width );
     SetDWBE( &p_buffer[14], p_input->p_fmt->video.i_sar_den );/* 24 bits, 15~ */
     SetDWBE( &p_buffer[11], p_input->p_fmt->video.i_sar_num );/* 24 bits, 12~ */
     SetWBE( &p_buffer[10], p_input->p_fmt->video.i_height );
-    SetDWBE( &p_buffer[18], p_input->p_fmt->video.i_frame_rate );
-    SetDWBE( &p_buffer[22], p_input->p_fmt->video.i_frame_rate_base );
+    SetDWBE( &p_buffer[18], p_stream->fmt.video.i_frame_rate );
+    SetDWBE( &p_buffer[22], p_stream->fmt.video.i_frame_rate_base );
 }
 
 static bool OggCreateHeaders( sout_mux_t *p_mux )
@@ -997,25 +1002,25 @@ static bool OggCreateHeaders( sout_mux_t *p_mux )
             sout_input_t *p_input = p_mux->pp_inputs[i];
             p_stream = (ogg_stream_t*)p_input->p_sys;
 
-            bool video = ( p_stream->i_fourcc == VLC_CODEC_THEORA ||
-                           p_stream->i_fourcc == VLC_CODEC_DIRAC ||
-                           p_stream->i_fourcc == VLC_CODEC_DAALA );
+            bool video = ( p_stream->fmt.i_codec == VLC_CODEC_THEORA ||
+                           p_stream->fmt.i_codec == VLC_CODEC_DIRAC ||
+                           p_stream->fmt.i_codec == VLC_CODEC_DAALA );
             if( ( ( pass == 0 && !video ) || ( pass == 1 && video ) ) )
                 continue;
 
             msg_Dbg( p_mux, "creating header for %4.4s",
-                     (char *)&p_stream->i_fourcc );
+                     (char *)&p_stream->fmt.i_codec );
 
             ogg_stream_init( &p_stream->os, p_stream->i_serial_no );
             p_stream->b_new = false;
             p_stream->i_packet_no = 0;
             p_stream->b_started = true;
 
-            if( p_stream->i_fourcc == VLC_CODEC_VORBIS ||
-                p_stream->i_fourcc == VLC_CODEC_SPEEX ||
-                p_stream->i_fourcc == VLC_CODEC_OPUS ||
-                p_stream->i_fourcc == VLC_CODEC_THEORA ||
-                p_stream->i_fourcc == VLC_CODEC_DAALA )
+            if( p_stream->fmt.i_codec == VLC_CODEC_VORBIS ||
+                p_stream->fmt.i_codec == VLC_CODEC_SPEEX ||
+                p_stream->fmt.i_codec == VLC_CODEC_OPUS ||
+                p_stream->fmt.i_codec == VLC_CODEC_THEORA ||
+                p_stream->fmt.i_codec == VLC_CODEC_DAALA )
             {
                 /* First packet in order: vorbis/speex/opus/theora/daala info */
                 unsigned pi_size[XIPH_MAX_HEADER_COUNT];
@@ -1042,14 +1047,14 @@ static bool OggCreateHeaders( sout_mux_t *p_mux )
                 p_og = OggStreamFlush( p_mux, &p_stream->os, 0 );
 
                 /* Get keyframe_granule_shift for theora or daala granulepos calculation */
-                if( p_stream->i_fourcc == VLC_CODEC_THEORA ||
-                    p_stream->i_fourcc == VLC_CODEC_DAALA )
+                if( p_stream->fmt.i_codec == VLC_CODEC_THEORA ||
+                    p_stream->fmt.i_codec == VLC_CODEC_DAALA )
                 {
                     p_stream->i_keyframe_granule_shift =
                         ( (op.packet[40] & 0x03) << 3 ) | ( (op.packet[41] & 0xe0) >> 5 );
                 }
             }
-            else if( p_stream->i_fourcc == VLC_CODEC_DIRAC )
+            else if( p_stream->fmt.i_codec == VLC_CODEC_DIRAC )
             {
                 op.packet = p_input->p_fmt->p_extra;
                 op.bytes  = p_input->p_fmt->i_extra;
@@ -1060,7 +1065,7 @@ static bool OggCreateHeaders( sout_mux_t *p_mux )
                 ogg_stream_packetin( &p_stream->os, &op );
                 p_og = OggStreamFlush( p_mux, &p_stream->os, 0 );
             }
-            else if( p_stream->i_fourcc == VLC_CODEC_FLAC )
+            else if( p_stream->fmt.i_codec == VLC_CODEC_FLAC )
             {
                 /* flac stream marker (yeah, only that in the 1st packet) */
                 op.packet = (unsigned char *)"fLaC";
@@ -1072,7 +1077,7 @@ static bool OggCreateHeaders( sout_mux_t *p_mux )
                 ogg_stream_packetin( &p_stream->os, &op );
                 p_og = OggStreamFlush( p_mux, &p_stream->os, 0 );
             }
-            else if( p_stream->i_fourcc == VLC_CODEC_VP8 )
+            else if( p_stream->fmt.i_codec == VLC_CODEC_VP8 )
             {
                 /* VP8 Header */
                 op.packet = malloc( 26 );
@@ -1094,7 +1099,7 @@ static bool OggCreateHeaders( sout_mux_t *p_mux )
                 op.packet = malloc( sizeof(*p_stream->p_oggds_header) + p_stream->p_oggds_header->i_size );
                 if( !op.packet )
                     return false;
-                op.bytes  = OggFillDsHeader( op.packet, p_stream->p_oggds_header, p_stream->i_cat );
+                op.bytes  = OggFillDsHeader( op.packet, p_stream->p_oggds_header, p_stream->fmt.i_cat );
                 op.b_o_s  = 1;
                 op.e_o_s  = 0;
                 op.granulepos = 0;
@@ -1178,11 +1183,11 @@ static bool OggCreateHeaders( sout_mux_t *p_mux )
         sout_input_t *p_input = p_mux->pp_inputs[i];
         ogg_stream_t *p_stream = (ogg_stream_t*)p_input->p_sys;
 
-        if( p_stream->i_fourcc == VLC_CODEC_VORBIS ||
-            p_stream->i_fourcc == VLC_CODEC_SPEEX ||
-            p_stream->i_fourcc == VLC_CODEC_OPUS ||
-            p_stream->i_fourcc == VLC_CODEC_THEORA ||
-            p_stream->i_fourcc == VLC_CODEC_DAALA )
+        if( p_stream->fmt.i_codec == VLC_CODEC_VORBIS ||
+            p_stream->fmt.i_codec == VLC_CODEC_SPEEX ||
+            p_stream->fmt.i_codec == VLC_CODEC_OPUS ||
+            p_stream->fmt.i_codec == VLC_CODEC_THEORA ||
+            p_stream->fmt.i_codec == VLC_CODEC_DAALA )
         {
             unsigned pi_size[XIPH_MAX_HEADER_COUNT];
             void     *pp_data[XIPH_MAX_HEADER_COUNT];
@@ -1214,8 +1219,8 @@ static bool OggCreateHeaders( sout_mux_t *p_mux )
                     block_ChainAppend( &p_hdr, p_og );
             }
         }
-        else if( p_stream->i_fourcc != VLC_CODEC_FLAC &&
-                 p_stream->i_fourcc != VLC_CODEC_DIRAC )
+        else if( p_stream->fmt.i_codec != VLC_CODEC_FLAC &&
+                 p_stream->fmt.i_codec != VLC_CODEC_DIRAC )
         {
             uint8_t com[128];
             int     i_com;
@@ -1237,8 +1242,8 @@ static bool OggCreateHeaders( sout_mux_t *p_mux )
         }
 
         /* Special case for mp4v and flac */
-        if( ( p_stream->i_fourcc == VLC_CODEC_MP4V ||
-              p_stream->i_fourcc == VLC_CODEC_FLAC ) &&
+        if( ( p_stream->fmt.i_codec == VLC_CODEC_MP4V ||
+              p_stream->fmt.i_codec == VLC_CODEC_FLAC ) &&
             p_input->p_fmt->i_extra )
         {
             /* Send a packet with the VOL data for mp4v
@@ -1247,7 +1252,7 @@ static bool OggCreateHeaders( sout_mux_t *p_mux )
             op.bytes  = p_input->p_fmt->i_extra;
             op.packet = p_input->p_fmt->p_extra;
             uint8_t flac_streaminfo[34 + 4];
-            if( p_stream->i_fourcc == VLC_CODEC_FLAC )
+            if( p_stream->fmt.i_codec == VLC_CODEC_FLAC )
             {
                 if (op.bytes == 42 && !memcmp(op.packet, "fLaC", 4)) {
                     op.bytes -= 4;
@@ -1427,13 +1432,13 @@ static bool AllocateIndex( sout_mux_t *p_mux, sout_input_t *p_input )
         uint64_t i;
 
         if( p_input->p_fmt->i_cat == VIDEO_ES &&
-                p_input->p_fmt->video.i_frame_rate )
+                p_stream->fmt.video.i_frame_rate )
         {
             /* optimize for fps < 1 */
             i_interval= __MAX( p_mux->p_sys->skeleton.i_index_intvl * 1000,
                        INT64_C(10000000) *
-                       p_input->p_fmt->video.i_frame_rate_base /
-                       p_input->p_fmt->video.i_frame_rate );
+                       p_stream->fmt.video.i_frame_rate_base /
+                       p_stream->fmt.video.i_frame_rate );
         }
 
         size_t i_tuple_size = 0;
@@ -1580,20 +1585,20 @@ static int MuxBlock( sout_mux_t *p_mux, sout_input_t *p_input )
     ogg_packet op;
     uint64_t i_time;
 
-    if( p_stream->i_fourcc != VLC_CODEC_VORBIS &&
-        p_stream->i_fourcc != VLC_CODEC_FLAC &&
-        p_stream->i_fourcc != VLC_CODEC_SPEEX &&
-        p_stream->i_fourcc != VLC_CODEC_OPUS &&
-        p_stream->i_fourcc != VLC_CODEC_THEORA &&
-        p_stream->i_fourcc != VLC_CODEC_DAALA &&
-        p_stream->i_fourcc != VLC_CODEC_VP8 &&
-        p_stream->i_fourcc != VLC_CODEC_DIRAC )
+    if( p_stream->fmt.i_codec != VLC_CODEC_VORBIS &&
+        p_stream->fmt.i_codec != VLC_CODEC_FLAC &&
+        p_stream->fmt.i_codec != VLC_CODEC_SPEEX &&
+        p_stream->fmt.i_codec != VLC_CODEC_OPUS &&
+        p_stream->fmt.i_codec != VLC_CODEC_THEORA &&
+        p_stream->fmt.i_codec != VLC_CODEC_DAALA &&
+        p_stream->fmt.i_codec != VLC_CODEC_VP8 &&
+        p_stream->fmt.i_codec != VLC_CODEC_DIRAC )
     {
         p_data = block_Realloc( p_data, 1, p_data->i_buffer );
         p_data->p_buffer[0] = PACKET_IS_SYNCPOINT;      // FIXME
     }
 
-    if ( p_stream->i_fourcc == VLC_CODEC_DIRAC && p_stream->i_baseptsdelay < 0 )
+    if ( p_stream->fmt.i_codec == VLC_CODEC_DIRAC && p_stream->i_baseptsdelay < 0 )
         p_stream->i_baseptsdelay = p_data->i_pts - p_data->i_dts;
 
     op.packet   = p_data->p_buffer;
@@ -1603,12 +1608,12 @@ static int MuxBlock( sout_mux_t *p_mux, sout_input_t *p_input )
     op.packetno = p_stream->i_packet_no++;
     op.granulepos = -1;
 
-    if( p_stream->i_cat == AUDIO_ES )
+    if( p_stream->fmt.i_cat == AUDIO_ES )
     {
-        if( p_stream->i_fourcc == VLC_CODEC_VORBIS ||
-            p_stream->i_fourcc == VLC_CODEC_FLAC ||
-            p_stream->i_fourcc == VLC_CODEC_OPUS ||
-            p_stream->i_fourcc == VLC_CODEC_SPEEX )
+        if( p_stream->fmt.i_codec == VLC_CODEC_VORBIS ||
+            p_stream->fmt.i_codec == VLC_CODEC_FLAC ||
+            p_stream->fmt.i_codec == VLC_CODEC_OPUS ||
+            p_stream->fmt.i_codec == VLC_CODEC_SPEEX )
         {
             /* number of sample from begining + current packet */
             op.granulepos =
@@ -1625,10 +1630,10 @@ static int MuxBlock( sout_mux_t *p_mux, sout_input_t *p_input )
                 p_stream->p_oggds_header->i_samples_per_unit / CLOCK_FREQ;
         }
     }
-    else if( p_stream->i_cat == VIDEO_ES )
+    else if( p_stream->fmt.i_cat == VIDEO_ES )
     {
-        if( p_stream->i_fourcc == VLC_CODEC_THEORA ||
-            p_stream->i_fourcc == VLC_CODEC_DAALA )
+        if( p_stream->fmt.i_codec == VLC_CODEC_THEORA ||
+            p_stream->fmt.i_codec == VLC_CODEC_DAALA )
         {
             p_stream->i_num_frames++;
             if( p_data->i_flags & BLOCK_FLAG_TYPE_I )
@@ -1638,14 +1643,15 @@ static int MuxBlock( sout_mux_t *p_mux, sout_input_t *p_input )
 
                 /* presentation time */
                 i_time = CLOCK_FREQ * ( p_stream->i_num_frames - 1 ) *
-                         p_input->p_fmt->video.i_frame_rate_base /  p_input->p_fmt->video.i_frame_rate;
+                        p_stream->fmt.video.i_frame_rate_base /
+                        p_stream->fmt.video.i_frame_rate;
                 AddIndexEntry( p_mux, i_time, p_input );
             }
 
             op.granulepos = (p_stream->i_last_keyframe << p_stream->i_keyframe_granule_shift )
                           | (p_stream->i_num_frames-p_stream->i_last_keyframe);
         }
-        else if( p_stream->i_fourcc == VLC_CODEC_DIRAC )
+        else if( p_stream->fmt.i_codec == VLC_CODEC_DIRAC )
         {
 
 #define FRAME_ROUND(a) \
@@ -1653,10 +1659,12 @@ static int MuxBlock( sout_mux_t *p_mux, sout_input_t *p_input )
         a += 5000;\
     a /= CLOCK_FREQ;
 
-            mtime_t dt = (p_data->i_dts - p_sys->i_start_dts) * p_input->p_fmt->video.i_frame_rate / p_input->p_fmt->video.i_frame_rate_base;
+            mtime_t dt = (p_data->i_dts - p_sys->i_start_dts) * p_stream->fmt.video.i_frame_rate /
+                    p_stream->fmt.video.i_frame_rate_base;
             FRAME_ROUND( dt );
 
-            mtime_t pt = (p_data->i_pts - p_sys->i_start_dts - p_stream->i_baseptsdelay ) * p_input->p_fmt->video.i_frame_rate / p_input->p_fmt->video.i_frame_rate_base;
+            mtime_t pt = (p_data->i_pts - p_sys->i_start_dts - p_stream->i_baseptsdelay ) *
+                    p_stream->fmt.video.i_frame_rate / p_stream->fmt.video.i_frame_rate_base;
             FRAME_ROUND( pt );
 
             /* (shro) some PTS could be repeated within 1st frames */
@@ -1696,7 +1704,7 @@ static int MuxBlock( sout_mux_t *p_mux, sout_input_t *p_input )
 
             AddIndexEntry( p_mux, dt, p_input );
         }
-        else if( p_stream->i_fourcc == VLC_CODEC_VP8 )
+        else if( p_stream->fmt.i_codec == VLC_CODEC_VP8 )
         {
             p_stream->i_num_frames++;
             if( p_data->i_flags & BLOCK_FLAG_TYPE_I )
@@ -1706,7 +1714,7 @@ static int MuxBlock( sout_mux_t *p_mux, sout_input_t *p_input )
 
                 /* presentation time */
                 i_time = CLOCK_FREQ * ( p_stream->i_num_frames - 1 ) *
-                         p_input->p_fmt->video.i_frame_rate_base /  p_input->p_fmt->video.i_frame_rate;
+                         p_stream->fmt.video.i_frame_rate_base / p_stream->fmt.video.i_frame_rate;
                 AddIndexEntry( p_mux, i_time, p_input );
             }
             op.granulepos = ( ((int64_t)p_stream->i_num_frames) << 32 ) |
@@ -1716,7 +1724,7 @@ static int MuxBlock( sout_mux_t *p_mux, sout_input_t *p_input )
             op.granulepos = ( p_data->i_dts - p_sys->i_start_dts ) * INT64_C(10) /
                 p_stream->p_oggds_header->i_time_unit;
     }
-    else if( p_stream->i_cat == SPU_ES )
+    else if( p_stream->fmt.i_cat == SPU_ES )
     {
         /* granulepos is in millisec */
         op.granulepos = ( p_data->i_dts - p_sys->i_start_dts ) / 1000;
@@ -1727,9 +1735,9 @@ static int MuxBlock( sout_mux_t *p_mux, sout_input_t *p_input )
     p_stream->u_last_granulepos = op.granulepos;
     ogg_stream_packetin( &p_stream->os, &op );
 
-    if( p_stream->i_cat == SPU_ES ||
-        p_stream->i_fourcc == VLC_CODEC_SPEEX ||
-        p_stream->i_fourcc == VLC_CODEC_DIRAC )
+    if( p_stream->fmt.i_cat == SPU_ES ||
+        p_stream->fmt.i_codec == VLC_CODEC_SPEEX ||
+        p_stream->fmt.i_codec == VLC_CODEC_DIRAC )
     {
         /* Subtitles or Speex packets are quite small so they
          * need to be flushed to be sent on time */
